@@ -151,8 +151,9 @@ static int prlimit(pid_t p, int resource,
 }
 #endif
 
-static void __attribute__ ((__noreturn__)) usage(FILE * out)
+static void __attribute__((__noreturn__)) usage(void)
 {
+	FILE *out = stdout;
 	size_t i;
 
 	fputs(USAGE_HEADER, out);
@@ -171,8 +172,8 @@ static void __attribute__ ((__noreturn__)) usage(FILE * out)
 		"     --noheadings       don't print headings\n"
 		"     --raw              use the raw output format\n"
 		"     --verbose          verbose output\n"
-		" -h, --help             display this help and exit\n"
-		" -V, --version          output version information and exit\n"), out);
+		), out);
+	printf(USAGE_HELP_OPTIONS(24));
 
 	fputs(_("\nResources Options:\n"), out);
 	fputs(_(" -c, --core             maximum size of core files created\n"
@@ -193,14 +194,13 @@ static void __attribute__ ((__noreturn__)) usage(FILE * out)
 		" -y, --rttime           CPU time in microseconds a process scheduled\n"
 		"                        under real-time scheduling\n"), out);
 
-	fputs(_("\nAvailable columns (for --output):\n"), out);
-
+	fputs(USAGE_COLUMNS, out);
 	for (i = 0; i < ARRAY_SIZE(infos); i++)
 		fprintf(out, " %11s  %s\n", infos[i].name, _(infos[i].help));
 
-	fprintf(out, USAGE_MAN_TAIL("prlimit(1)"));
+	printf(USAGE_MAN_TAIL("prlimit(1)"));
 
-	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
+	exit(EXIT_SUCCESS);
 }
 
 static inline int get_column_id(int num)
@@ -397,7 +397,9 @@ static int get_range(char *str, rlim_t *soft, rlim_t *hard, int *found)
 		*found |= PRLIMIT_SOFT | PRLIMIT_HARD;
 		return 0;
 
-	} else if (*str == ':') {			/* <:hard> */
+	}
+
+	if (*str == ':') {			/* <:hard> */
 		str++;
 
 		if (strcmp(str, INFINITY_STR) != 0) {
@@ -516,7 +518,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	INIT_LIST_HEAD(&lims);
 
@@ -584,8 +586,6 @@ int main(int argc, char **argv)
 				errx(EXIT_FAILURE, _("option --pid may be specified only once"));
 			pid = strtos32_or_err(optarg, _("invalid PID argument"));
 			break;
-		case 'h':
-			usage(stdout);
 		case 'o':
 			ncolumns = string_to_idarray(optarg,
 						     columns, ARRAY_SIZE(columns),
@@ -593,10 +593,6 @@ int main(int argc, char **argv)
 			if (ncolumns < 0)
 				return EXIT_FAILURE;
 			break;
-		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
-
 		case NOHEADINGS_OPTION:
 			no_headings = 1;
 			break;
@@ -606,6 +602,11 @@ int main(int argc, char **argv)
 		case RAW_OPTION:
 			raw = 1;
 			break;
+
+		case 'h':
+			usage();
+		case 'V':
+			print_version(EXIT_SUCCESS);
 		default:
 			errtryhelp(EXIT_FAILURE);
 		}
@@ -639,7 +640,7 @@ int main(int argc, char **argv)
 	if (argc > optind) {
 		/* prlimit [options] COMMAND */
 		execvp(argv[optind], &argv[optind]);
-		err(EXIT_FAILURE, _("failed to execute %s"), argv[optind]);
+		errexec(argv[optind]);
 	}
 
 	return EXIT_SUCCESS;
